@@ -58,8 +58,30 @@ void mysqlx_stmt_t::init_data_model()
     default:
       m_data_model = cdk::protocol::mysqlx::DEFAULT;
       m_parser_mode = parser::Parser_mode::TABLE;
-      break;
+    break;
   }
+  m_group_by_list.set_parser_mode(m_parser_mode);
+}
+
+void mysqlx_stmt_t::copy_parent_data(mysqlx_stmt_struct *parent)
+{
+  if (parent->m_where.get())
+    m_where.reset(new Expression_parser(*static_cast<parser::Expression_parser*>(parent->m_where.get())));
+
+  if (parent->m_having.get())
+    m_having.reset(new Expression_parser(*static_cast<parser::Expression_parser*>(parent->m_having.get())));
+
+  if (parent->m_limit.get())
+    m_limit.reset(new Limit(*parent->m_limit.get()));
+
+  if (parent->m_order_by.get())
+    m_order_by.reset(new Order_by(*parent->m_order_by.get()));
+
+  if (parent->m_proj_list.get())
+    m_proj_list.reset(new Projection_list(*parent->m_proj_list.get()));
+
+    m_param_source = parent->m_param_source;
+    m_group_by_list = parent->m_group_by_list;
 }
 
 void mysqlx_stmt_t::init_crud()
@@ -474,11 +496,25 @@ mysqlx_result_t *mysqlx_stmt_t::exec()
   {
     case OP_SELECT:
       m_reply = sess.table_select(m_db_obj_ref,
+                                  NULL,               // view spec
                                   m_where.get(),
                                   m_proj_list.get(),
                                   m_order_by.get(),
-                                  NULL, // Group by
-                                  NULL, // Having
+                                  m_group_by_list.get_list(),
+                                  m_having.get(),
+                                  m_limit.get(),
+                                  m_param_source.count() ? &m_param_source : NULL);
+      break;
+    case OP_VIEW_CREATE:
+    case OP_VIEW_UPDATE:
+    case OP_VIEW_REPLACE:
+      m_reply = sess.table_select(m_db_obj_ref,
+                                  &m_view_spec,
+                                  m_where.get(),
+                                  m_proj_list.get(),
+                                  m_order_by.get(),
+                                  m_group_by_list.get_list(),
+                                  m_having.get(),
                                   m_limit.get(),
                                   m_param_source.count() ? &m_param_source : NULL);
       break;
@@ -514,11 +550,12 @@ mysqlx_result_t *mysqlx_stmt_t::exec()
       break;
     case OP_FIND:
       m_reply = sess.coll_find(m_db_obj_ref,
+                               NULL,                // view spec
                                m_where.get(),
                                m_proj_list.get(),
                                m_order_by.get(),
-                               NULL, // Group by
-                               NULL, // Having
+                               m_group_by_list.get_list(),
+                               m_having.get(),
                                m_limit.get(),
                                m_param_source.count() ? &m_param_source : NULL);
       break;
@@ -582,7 +619,32 @@ mysqlx_result_t *mysqlx_stmt_t::exec()
 }
 
 /*
-  Set WHERE for CRUD operation
+  Set member to a CDK expression which results from parsing given string val
+*/
+int mysqlx_stmt_t::set_expression(cdk::scoped_ptr<cdk::Expression> &member,
+                                  const char *val)
+{
+  if (!val || !val[0])
+    return RESULT_OK;
+
+  int res = RESULT_OK;
+  try
+  {
+    member.reset(new Expression_parser(m_parser_mode, val));
+    if (!member.get())
+      res = RESULT_ERROR;
+  }
+  catch (...)
+  {
+    res = RESULT_ERROR;
+    // TODO: Get exception details
+  }
+  return res;
+}
+
+
+/*
+  Set WHERE for statement operation
   PARAMETERS:
     where_expr - character string containing WHERE clause,
                  which will be parsed as required
@@ -595,22 +657,28 @@ mysqlx_result_t *mysqlx_stmt_t::exec()
 */
 int mysqlx_stmt_t::set_where(const char *where_expr)
 {
-  if (!where_expr || !where_expr[0])
-    return RESULT_OK;
+  if (m_op_type == OP_ADD || m_op_type == OP_INSERT)
+    throw Mysqlx_exception(MYSQLX_ERROR_OP_NOT_SUPPORTED);
+  return set_expression(m_where, where_expr);
+}
 
-  int res = RESULT_OK;
-  try
-  {
-    m_where.reset(new Expression_parser(m_parser_mode, where_expr));
-    if (!m_where.get())
-      res = RESULT_ERROR;
-  }
-  catch(...)
-  {
-    res = RESULT_ERROR;
-    // TODO: Get exception details
-  }
-  return res;
+/*
+  Set HAVING for statement operation
+  PARAMETERS:
+    having_expr - character string containing HAVING clause,
+                  which will be parsed as required
+
+  RETURN:
+    RESULT_OK - on success
+    RESULT_ERROR - on error
+
+  NOTE: each call to this function replaces previously set HAVING
+*/
+int mysqlx_stmt_t::set_having(const char *having_expr)
+{
+  if (m_op_type != OP_SELECT && m_op_type != OP_FIND)
+    throw Mysqlx_exception(MYSQLX_ERROR_OP_NOT_SUPPORTED);
+  return set_expression(m_having, having_expr);
 }
 
 /*
@@ -627,6 +695,9 @@ int mysqlx_stmt_t::set_where(const char *where_expr)
 */
 int mysqlx_stmt_t::set_limit(row_count_t row_count, row_count_t offset)
 {
+  if (m_op_type == OP_ADD || m_op_type == OP_INSERT)
+    throw Mysqlx_exception(MYSQLX_ERROR_OP_NOT_SUPPORTED);
+
   int res = RESULT_OK;
   try
   {
@@ -713,6 +784,88 @@ void mysqlx_stmt_t::clear_order_by()
   if(m_order_by.get()) m_order_by->clear();
 }
 
+bool mysqlx_stmt_t::is_view_op()
+{
+  return (m_op_type == OP_VIEW_CREATE || m_op_type == OP_VIEW_UPDATE ||
+          m_op_type == OP_VIEW_REPLACE);
+}
+
+void mysqlx_stmt_t::set_view_algorithm(int algorithm)
+{
+  if (!is_view_op())
+    throw Mysqlx_exception(MYSQLX_ERROR_VIEW_TYPE_MSG);
+
+  m_view_spec.set_algorithm(algorithm);
+}
+
+void mysqlx_stmt_t::set_view_security(int security)
+{
+  if (!is_view_op())
+    throw Mysqlx_exception(MYSQLX_ERROR_VIEW_TYPE_MSG);
+
+  m_view_spec.set_security(security);
+}
+
+void mysqlx_stmt_t::set_view_check_option(int option)
+{
+  if (!is_view_op())
+    throw Mysqlx_exception(MYSQLX_ERROR_VIEW_TYPE_MSG);
+
+  m_view_spec.set_check(option);
+}
+
+void mysqlx_stmt_t::set_view_definer(const char* user)
+{
+  if (!is_view_op())
+    throw Mysqlx_exception(MYSQLX_ERROR_VIEW_TYPE_MSG);
+
+  m_view_spec.set_definer(user);
+}
+
+void mysqlx_stmt_t::set_view_columns(va_list args)
+{
+  if (!is_view_op())
+    throw Mysqlx_exception(MYSQLX_ERROR_VIEW_TYPE_MSG);
+
+  m_view_spec.set_columns(args);
+}
+
+void mysqlx_stmt_t::set_view_properties(va_list args)
+{
+  int64_t view_option = 0;
+  if (!is_view_op())
+    throw Mysqlx_exception(MYSQLX_ERROR_VIEW_TYPE_MSG);
+
+  /*
+    The list of arguments is ending after the list of columns,
+    so the function must stop getting more arguments
+  */
+  while (view_option != VIEW_OPTION_COLUMNS &&
+         (view_option = (int64_t)va_arg(args, void*)) != 0)
+  {
+    switch (view_option)
+    {
+      case VIEW_OPTION_ALGORITHM:
+        set_view_algorithm(va_arg(args, int));
+      break;
+      case VIEW_OPTION_SECURITY:
+        set_view_security(va_arg(args, int));
+      break;
+      case VIEW_OPTION_CHECK_OPTION:
+        set_view_check_option(va_arg(args, int));
+      break;
+      case VIEW_OPTION_DEFINER:
+        set_view_definer(va_arg(args, const char*));
+      break;
+      case VIEW_OPTION_COLUMNS:
+        set_view_columns(args);
+      break;
+      default:
+        throw Mysqlx_exception("Wrong VIEW property");
+    }
+  }
+}
+
 mysqlx_stmt_t::~mysqlx_stmt_struct()
 {
   try
@@ -728,7 +881,7 @@ int mysqlx_stmt_t::add_document(const char *json_doc)
   int res = RESULT_OK;
   if (m_op_type != OP_ADD)
   {
-    m_error.set("Wrong operation type. Only ADD is supported.", 0);
+    set_diagnostic("Wrong operation type. Only ADD is supported.", 0);
     return RESULT_ERROR;
   }
 
@@ -763,4 +916,34 @@ int mysqlx_stmt_t::add_multiple_documents(va_list args)
     rc = add_document(json_doc);
   }
   return rc;
-}
\ No newline at end of file
+}
+
+int mysqlx_stmt_t::add_group_by(va_list args)
+{
+  m_group_by_list.clear();
+  if (m_op_type != OP_SELECT && m_op_type != OP_FIND)
+  {
+    set_diagnostic(MYSQLX_ERROR_OP_NOT_SUPPORTED, 0);
+    return RESULT_ERROR;
+  }
+
+  const char *group_by;
+  while ((group_by = va_arg(args, char*)) != NULL)
+  {
+    m_group_by_list.add_group_by(group_by);
+  }
+  return RESULT_OK;
+}
+
+void Group_by_list::process(cdk::Expr_list::Processor& prc) const
+{
+  prc.list_begin();
+
+  for (group_by_list_type::const_iterator it = m_group_by_list.begin();
+    it != m_group_by_list.end(); ++it)
+  {
+    parser::Expression_parser expr_parser(m_parser_mode, *it);
+    expr_parser.process_if(prc.list_el());
+  }
+  prc.list_end();
+}
